//
//  EDStatisticsReport.h
//  EDStatisticsReport
//
//  Created by Arnaud Thiercelin on 2/3/12.
//  Copyright (c) 2012 Flying Pig. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <EDStatisticsReport/EDStatisticsReportOwnerProtocol.h>
#import <EDStatisticsReport/EDStatisticsReportColumnConfiguration.h>

//default ids
#define EDStatisticsReportFatherSonItemIdKey		@"id"
#define EDStatisticsReportFatherSonParentIdKey		@"id_parent"

@interface EDStatisticsReport : NSObject
{	
	NSString									*_name;
	NSString									*_description;
	NSString									*_maker;
	NSString									*_version;
	NSString									*_minEdouardVersion;
	BOOL                                        _enabled;
	// column config in properties
	
	id <EDStatisticsReportOwnerProtocol>		_owner;
		
	// Config
	IBOutlet NSView				*_configSheet;
	
	// Graph Rendering
	BOOL						_rendersAGraph;
	NSArray					*_graphAvailableIdentifiers;
	NSString					*_graphDefaultXAxisIdentifier;
	NSString					*_graphDefaultYAxisIdentifier;
	NSString					*_graphDefaultLabel; 
	
	// Data holding. Basically TV or OV
	NSMutableArray		*_standardData;
	
	BOOL						_usesFatherSonStructure;
	NSString					*_itemIdKey;
	NSString					*_fatherItemIdKey;
	NSString					*_outlineColumnIdentifier;
	NSMutableArray		*_fathersData;
	NSMutableArray		*_sonsData;
	
	// export
	NSArray						*_allowedFileTypes;
	IBOutlet NSView				*_accessoryView;
	
	
}

@property (retain) NSString									*name;
@property (retain) NSString									*description;
@property (retain) NSString									*maker;
@property (retain) NSString									*version;
@property (retain) NSString									*minEdouardVersion;
@property BOOL												enabled;
@property (retain) NSArray									*columnsConfiguration; // array of EDStatisticsReportColumnConfiguration
@property (assign) id <EDStatisticsReportOwnerProtocol>		owner;
@property (readonly) NSView									*configSheet;	
@property BOOL												rendersAGraph;
@property (readonly) NSArray						*graphAvailableIdentifiers;
@property (readonly) NSString					*graphDefaultXAxisIdentifier;
@property (readonly) NSString					*graphDefaultYAxisIdentifier;
@property (readonly) NSString					*graphDefaultLabel;
@property (retain) NSMutableArray							*standardData;
@property BOOL												usesFatherSonStructure;
@property (retain) NSString									*itemIdKey;
@property (retain) NSString									*fatherItemIdKey;
@property (retain) NSString									*outlineColumnIdentifier;
@property (retain) NSMutableArray							*fathersData;
@property (retain) NSMutableArray							*sonsData;
@property (readonly) NSArray								*allowedFileTypes;
@property BOOL												exportHeaders;
@property (retain) NSView									*accessoryView;

- (void)load;
- (void)unload;

- (void)run;
- (void)exportToFile:(NSURL *)exportFile;

#pragma mark ConfigView Display
// These are for the owner to notify the report it loaded the configSheet.
// This is useful when the config view need to pull data out of the database.
- (BOOL)configSheetShouldDisplay;
- (void)configSheetWillDisplay;
- (void)configSheetDidEndDisplay;

#pragma mark Functional Methods

- (NSArray *)getColumnsKeysInOrder;
- (NSArray *)getColumnsTitlesInOrder;

@end
