//
//  FPModelImageView.h
//  FPCoreGraphic.framework
//
//  Created by Flying Pig on 24/10/06.
//  Copyright © 2006 Flying Pig. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import <FPTemplates/FPShapesTypes.h>
#import <FPTemplates/FPModelShapeView.h>



//NSCoding Keys
#define FPModelImageViewDisplayResizeRectCodingKey								@"FPModelImageViewDisplayResizeRectCodingKey"
#define FPModelImageViewDisplayStrokeCodingKey									@"FPModelImageViewDisplayStrokeCodingKey"
#define FPModelImageViewDisplayFillCodingKey									@"FPModelImageViewDisplayFillCodingKey"
#define FPModelImageViewStrokeWidthCodingKey									@"FPModelImageViewStrokeWidthCodingKey"
#define FPModelImageViewShapeActionCodingKey									@"FPModelImageViewShapeActionCodingKey"
#define FPModelImageViewFillColorCodingKey										@"FPModelImageViewFillColorCodingKey"
#define FPModelImageViewStrokeColorCodingKey									@"FPModelImageViewStrokeColorCodingKey"
#define FPModelImageViewKeyCodingKey											@"FPModelImageViewKeyCodingKey"
#define FPModelImageViewTagCodingKey											@"FPModelImageViewTagCodingKey"

//Drag Operation Types
#define FPTemplatesImageViewDragOperationType										@"FPTemplatesImageViewDragOperationType"

//Default Values
#define FPModelImageViewStrokeWidthValue										1.0

//Events
#define FPModelImageViewDoubleClick												2

//Contextual Menu
#define FPModelImageViewContextMenuTitle										@"Menu"
#define FPModelImageViewContextMenuBringCloser									@"Bring Forward"
#define FPModelImageViewContextMenuSendBack										@"Send Backward"
#define FPModelImageViewContextMenuEdit											@"Edit"
#define FPModelImageViewContextMenuDelete										@"Delete"
#define FPModelImageViewContextMenuKeyEquivalent								@""	

//OpenPanel files
#define FPModelImageViewPNGFileType												@"png"
#define FPModelImageViewJPGFileType												@"jpg"
#define FPModelImageViewJPEGFileType											@"jpeg"
#define FPModelImageViewTIFFileType												@"tif"
#define FPModelImageViewTIFFFileType											@"tiff"
#define FPModelImageViewGIFFileType												@"gif"
#define FPModelImageViewPICTFileType											@"pict"
#define FPModelImageViewPCTFileType												@"pct"
#define FPModelImageViewPICFileType												@"pic"
#define FPModelImageViewBMPFileType												@"bmp"
#define FPModelImageViewICOFileType												@"ico"
#define FPModelImageViewICNSFileType											@"icns"




@class FPModelImageTag;

@interface FPModelImageView : NSImageView <NSCoding, FPModelShapeViewProtocol, FPModelShapeViewTagProtocol>
{
	
@private
	NSPoint								_clicPoint;
	NSPoint								_lastPoint;		
	BOOL								_displayResizeRect;
	BOOL								_currentEditedShape;
	BOOL								_displayStroke;
	float								_strokeWidth;
	NSColor								*_strokeColor;
	FPShapesActions						_shapeAction;
	NSColor								*_fillColor;
	BOOL								_displayFill;	
	NSMenu								*_contextMenu;
	BOOL								_isBeingEdited;	
//	NSString							*_key;
	FPModelImageTag						*_modelTag;
}

//Properties
@property NSPoint						clicPoint;
@property NSPoint						lastPoint;
@property BOOL							displayResizeRect;
@property BOOL							currentEditedShape;
@property BOOL							displayStroke;
@property float							strokeWidth;
@property(retain) NSColor				*strokeColor;
@property FPShapesActions				shapeAction;
@property(retain) NSColor				*fillColor;
@property BOOL							displayFill;
@property(retain) NSMenu				*contextMenu;
@property BOOL							isBeingEdited;
//@property(retain) NSString				*key;
@property(retain) FPModelImageTag		*modelTag;


//Open Image Panel
- (void)openImagePanel;
- (void)openPanelDidEnd:(NSOpenPanel *)panel returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;

//Setters
- (void)setIsBeingEdited:(BOOL)value;
- (void)setCurrentEditedShape:(BOOL)value;
- (void)setModelTagWithDefaultValues:(FPModelImageTag *)newModelImageTag;

@end
