//
//  FPModelTableView.h
//  FPCoreGraphic.framework
//
//  Created by Flying Pig on 4/20/07.
//  Copyright © 2007 Flying Pig. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import <FPTemplates/FPShapesTypes.h>


//NSCoding Keys
#define FPModelTableViewDisplayStrokeCodingKey								 	@"FPModelTableViewDisplayStrokeCodingKey"
#define FPModelTableViewStrokeColorCodingKey							 		@"FPModelTableViewStrokeColorCodingKey"
#define FPModelTableViewStrokeWidthCodingKey							 		@"FPModelTableViewStrokeWidthCodingKey"
#define FPModelTableViewFillColorCodingKey								 		@"FPModelTableViewFillColorCodingKey"
#define FPModelTableViewDisplayResizeRectCodingKey								@"FPModelTableViewDisplayResizeRectCodingKey"
#define FPModelTableViewDisplayFillCodingKey									@"FPModelTableViewDisplayFillCodingKey"
#define FPModelTableViewKeyCodingKey											@"FPModelTableViewKeyCodingKey"
#define FPModelTableViewAlternativeRowColorCodingKey							@"FPModelTableViewAlternativeRowColorCodingKey"
#define FPModelTableViewDataSourceCodingKey										@"FPModelTableViewDataSourceCodingKey"
#define FPModelTableViewDividingDataSourceCodingKey								@"FPModelTableViewDividingDataSourceCodingKey"
#define FPTemplatesTableViewTagCodingKey @"FPTemplatesTableViewTagCodingKey"

//Default Values
#define FPModelTableViewStrokeWidthValue										1.0
#define FPModelTableViewNoSelectedColumnIndex									-1
#define FPModelTableViewNoSelectedRowIndex										-1
#define FPModelTableViewNoSelectedRowIndex										-1
#define FPModelTableViewColumnWidth												100

//Columns
#define FPModelTableViewColumn1Identifier										@"column1"
#define FPModelTableViewColumn1HeaderCellString									@"column1"
#define FPModelTableViewColumn2Identifier										@"column2"
#define FPModelTableViewColumn2HeaderCellString									@"column2"
#define FPModelTableViewColumn3Identifier										@"column3"
#define FPModelTableViewColumn3HeaderCellString									@"column3"
#define FPModelTableViewColumnIdentifier										@"column"
#define FPModelTableViewColumnHeaderCellString									@"column"
//Default Value
#define FPModelTableViewDataCellDefaultValue									@"---------"

//Contextual Menu
#define FPModelTableViewContextMenuDeleteRow									@"Delete the Row"
#define FPModelTableViewContextMenuDeleteColumn									@"Delete the Column"



@class FPModelTableHeaderView;
@class FPModelScrollView;
@class FPModelTableTag;

@interface FPModelTableView : NSTableView <NSCoding, FPModelShapeViewProtocol, FPModelShapeViewTagProtocol, NSTableViewDelegate> 
{
@private
	NSPoint									_clicPoint;
	NSPoint									_lastPoint;		
	BOOL									_displayResizeRect;
	BOOL									_displayStroke;
	BOOL									_displayFill;
	float									_strokeWidth;
	FPShapesActions							_shapeAction;
	NSColor									*_fillColor;
	NSColor									*_strokeColor;	
	NSMenu									*_contextMenu;
	NSString								*_key;
	NSColor									*_alternativeRowBackgroundColor;
	NSInteger								_lastSelectedColumnIndex;
	NSInteger								_lastSelectedRowIndex;
	NSInteger								_selectedRow;
	NSEvent									*_lastMouseDownEvent;
	FPModelScrollView						*_modelScrollView;
	FPModelTableTag							*_modelTag;
	BOOL									_printingMode;
}


//Properties
@property NSPoint							clicPoint;
@property NSPoint							lastPoint;		
@property BOOL								displayResizeRect;
@property BOOL								displayStroke;
@property BOOL								displayFill;
@property float								strokeWidth;
@property FPShapesActions					shapeAction;
@property(retain) NSColor					*fillColor;
@property(retain) NSColor					*strokeColor;
@property(retain) NSMenu					*contextMenu;
@property(retain) NSString					*key;
@property(retain) NSColor					*alternativeRowBackgroundColor;
@property NSInteger							lastSelectedColumnIndex;
@property NSInteger							lastSelectedRowIndex;
@property(retain) NSEvent					*lastMouseDownEvent;
@property(assign) FPModelScrollView			*modelScrollView;
@property(retain) FPModelTableTag			*modelTag;
@property BOOL								printingMode;


//Init
- (id)initWithScrollView:(FPModelScrollView *)scrollView;

//Setup
- (void)setupTableView;

//Setters
- (void)setModelTagWithDefaultValues:(FPModelTableTag *)newModelTableTag;

//Getters
- (FPModelTableHeaderView *)headerView;
- (FPModelTableTag *)modelTag;

@end
