/*
 *	FPShapesTypes.h
 *  FPCoreGraphic.framework
 *
 *  Created by Flying Pig on 29/09/06.
 *  Copyright © 2006 Flying Pig. All rights reserved.
 *
 */


//KeyCodes
typedef enum	FPModelViewKeyCode
	{
		FPModelViewReturnKeyCode = 36,
		FPModelViewEraseKeyCode = 51,
		FPModelViewEscKeyCode = 53,
		FPModelViewDeleteKeyCode = 117,
		FPModelViewLeftKeyCode = 123,
		FPModelViewRightKeyCode = 124,
		FPModelViewDownKeyCode = 125,
		FPModelViewUpKeyCode = 126
	}			FPModelViewKeyCode;



typedef enum	FPShapesTypes
	{
		FPNoneShape = 0,
		FPRectShape	= 1,
		FPCircleShape = 2,
		FPArcShape = 3,
		FPLineShape = 4,
		FPImageShape = 5,
		FPTableShape = 6,
		FPTextShape = 7
	}			FPShapesTypes;


typedef enum 	FPShapesActions
	{
		FPNoneAction = 0,
		FPResizeAction = 1,
		FPMoveAction = 2,
		FPVertTopResizeAction = 3,
		FPVertBotResizeAction = 4,
		FPHoriLeftResizeAction = 5,
		FPHoriRightResizeAction = 6,
		FPTopLeftResizeAction = 7,
		FPTopRightResizeAction = 8,
		FPBotLeftResizeAction = 9,
		FPBotRightResizeAction = 10,
		FPDoubleClicAction = 11
	}			FPShapesActions;


typedef	enum	FPLineMode
	{
		lineBLBR = 22,
		lineBLTR = 21,
		lineTLBR = 12,
		lineTLTR = 11
	}			FPLineMode;




//Use by the FPModelView to set shapeView's values. Each shapeView has to implement this protocol
@protocol FPModelShapeViewProtocol

//Getters
@required
- (NSPoint)clicPoint;
- (NSString *)key;

@optional
- (NSMenu *)contextMenu; 
- (NSPoint)lastPoint;
- (BOOL)displayResizeRect;
- (BOOL)currentEditedShape;
- (BOOL)displayStroke;
- (float)strokeWidth;
- (NSColor *)strokeColor;
- (FPShapesActions)shapeAction;
- (BOOL)displayFill;
- (NSColor *)fillColor;
- (NSImage *)image;
- (FPLineMode)mode;
- (NSInteger)resize;
- (BOOL)usesAlternatingRowBackgroundColors;
- (NSColor *)alternativeRowBackgroundColor;
- (NSUInteger)setGridStyleMask;
- (NSInteger)numberOfColumns;
- (NSInteger)numberOfRows;


//Setters
@required
- (void)setClicPoint:(NSPoint)clicPoint;
- (void)setKey:(NSString *)keyString;

// This need to be implemented to respond to dynamic data population from the setter in the root FPTemplateView
// When implemented this should perform the accurate calls depending on the specifics of the class.
// Ex: setDatasource for TV/OV, setString: for NSTextViews and setStringValue for Fields.
- (void)setShapeValue:(id)value;

@optional
- (void)setContextMenu:(NSMenu *)menu;
- (void)setLastPoint:(NSPoint)clicPoint;
- (void)setDisplayResizeRect:(BOOL)displayResizeRectBool;
- (void)setCurrentEditedShape:(BOOL)editedShapeBool;
- (void)setDisplayStroke:(BOOL)displayStrokeBool;
- (void)setStrokeWidth:(float)strokeWidthFloat;
- (void)setStrokeColor:(NSColor *)strokeColor;
- (void)setShapeAction:(FPShapesActions)shapeAction;
- (void)setDisplayFill:(BOOL)displayFillBool;
- (void)setFillColor:(NSColor *)fillColor;
- (void)setImage:(NSImage *)image;
- (void)setMode:(FPLineMode)lineMode;
- (void)setResize:(NSInteger)size;
- (void)setUsesAlternatingRowBackgroundColors:(BOOL)useAlternatingRowColors;
- (void)setAlternativeRowBackgroundColor:(NSColor *)color;
- (void)setGridStyleMask:(NSUInteger)gridType;
- (void)setNumberOfColumns:(NSInteger)number;
- (void)setNumberOfRows:(NSInteger)number;

@end



//Use to let us know if shapeViews can manage or not tags.
@class FPModelTag;

@protocol FPModelShapeViewTagProtocol

@property (retain) FPModelTag *modelTag;

- (void)setupShapeViewWithProfilesValues;

//- (FPModelTag *)modelTag;
//- (void)setModelTag:(FPModelTag *)newModelTag;

@end


