//
//  FPTemplatesView.h
//  FPCoreGraphic.framework
//
//  Created by Flying Pig on 26/09/06.
//  Copyright © 2006 Flying Pig. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <FPTemplates/FPShapesTypes.h>
#import <FPTemplates/FPTemplateViewControllerProtocol.h>

// To debug printing.
//#define DEBUG_PRINT

//NSCoding Keys
#define FPTemplatesViewModelFileKey												@"FPModelsViewModelFileKey"
#define FPTemplatesViewDrawFrameKey												@"FPModelsViewDrawFrameKey"
#define FPTemplatesViewDrawElemLayoutKey											@"FPModelsViewDrawElemLayoutKey"
#define FPTemplatesViewPrintInfoKey												@"FPModelsViewPrintInfoKey"

//Default Values
#define FPFrameThickness														1.0
#define FPShadowSize 															5
#define FPTemplateViewStrokeWidthDefaultValue										1.00

//Table Shape
#define FPTemplatesViewTableShapeNbColsColumns										@"nbColomns"										
#define FPTemplatesViewTableShapeNbRowsColumns										@"nbRows"
#define FPTemplatesViewTableShapeVertThickColumns									@"vertThick"
#define FPTemplatesViewTableShapeHoriThickColumns									@"horiThick"
#define FPTemplatesViewTableShapeShowHeadersColumns								@"showHeaders"

// Borders / Strokes
#define FPTemplatesViewNoBorderType		0
#define FPTemplatesViewLineBorderType	1

//Shadow
#define FPTemplatesViewShadowOffsetWidth											5.0
#define FPTemplatesViewShadowOffsetHeight											-5.0
#define FPTemplatesViewShadowBlurRadius											3.0
#define FPTemplatesViewShadowColorAlphaComponent									0.5	

//Notifications
#define FPTemplatesViewUserInfo													@"FPModelsViewUserInfo"
#define FPTemplatesViewMouseDidUpNotification										@"FPTemplatesViewMouseDidUpNotification"
#define FPTemplatesViewMouseDidDownNotification									@"FPTemplatesViewMouseDidDownNotification"
#define FPTemplatesViewTagDidChangeNotification									@"FPTemplatesViewTagDidChangeNotification"

@class FPModelsFile;
@class FPModelsGraphActionsView;
@class FPModelResizeView;


@interface FPTemplatesView : FPView <NSCoding>
{
@private
	
	//ModelsView's metadata
	FPModelsFile										*_modelFile;
	
	//View to manage events (it's always on top)
	FPModelsGraphActionsView							*_graphView;
	
	//Handle every resizing
	FPModelResizeView									*_modelResizeView;
	
	//Selected Shape
	NSView <FPModelShapeViewProtocol>					*_selectedShapeView;
	
	//PrintInfo
	NSPrintInfo											*_printInfo;
	
	//Zoom
	//float												_resizeRatio;

	//White Rect
	NSRect												_thePage;
	NSColor												*_pageColor;

	//_modelView's color
	NSColor												*_bgColor;
	
	//Shadow
	NSColor												*_shadowColor;

	
	//Mouse Events
	//MousseUp point
	NSPoint												_clicPoint;
	//MousseDragged point
	NSPoint												_mousePoint;
	NSPoint												_originStartPoint;
	NSUInteger											_mouseModifier;
		
	//Current object to draw
	FPShapesTypes										_elemSelected; 
	
	//Show shape's layout
	BOOL												_drawElemLayout;
	
	//Show page's layout
	BOOL												_drawFrame;
	
	//Area selection
	NSRect												_selectArea;
	
	//Printable Rect
	NSRect												_printedFrame;
	NSInteger											_pageNumber;
	
	//Image's ivars
	NSString											*_imgPath;
	
	//Table's ivars
	NSInteger											_nbCols;
	NSInteger											_nbRows;
	float												_vertThick;
	float												_horiThick;
	BOOL												_showHeaders;	
}


//Properties
@property (assign) NSObject <FPTemplateViewControllerProtocol> *controller;

@property(retain) FPModelsGraphActionsView				*graphView;
@property(readonly) FPModelsFile 						*modelFile;
@property(readonly) FPModelResizeView					*modelResizeView;
@property(assign) NSView <FPModelShapeViewProtocol>		*selectedShapeView;
@property FPShapesTypes									elemSelected;
@property NSRect										selectArea;
@property NSRect										printedFrame;
//@property(readonly) float								drawingZoom;
@property(readonly) NSRect								thePage;

// new gen of accessors - March 2013
@property (readonly) BOOL								showElementsFrames;
@property (readonly) BOOL								showPrintableArea;
@property (readonly) NSPrintInfo						*printInfo;

//Setters
- (void)setSelectedShapeView:(NSView <FPModelShapeViewProtocol> *)modelShapeView;

#pragma mark Getters allowing external resources to acces to shapeViews easily
- (NSArray *)getAllAvailableShapeIdentifiers;
- (NSView <FPModelShapeViewProtocol, FPModelShapeViewTagProtocol> *)modelShapeViewWithTagIdentifier:(NSString *)tagIdentifier;
- (NSArray *)modelAllShapeViewsWithTagIdentifier:(NSString *)tagIdentifier;
- (NSArray *)getAllShapeViewsWithSerialTagIdentifier:(NSString *)serialIdentifier;

#pragma mark Setters allowing external resources to acces to shapeViews easily
- (void)setValuesForAllShapesUsingDictionary:(NSDictionary *)dict;
- (void)setValue:(id)value forAllShapesWithIdentifier:(NSString *)identifier;

//Edit Current Subviews
- (void)eraseAllItems;
- (void)eraseSelectedItems;
- (void)unsetLastCurrentEditedShape;

//Resize Page
//- (void)resizePageForThePrinting;

#pragma mark Selections
- (NSArray *)getSelectedItems;
- (BOOL)isAnItemSelected;
//- (BOOL)performMultipleSelectionUsingSelectedArea:(NSRect)area;
- (BOOL)performSelection:(NSEvent *)theEvent;
- (void)selectSubView:(NSView <FPModelShapeViewProtocol>*)layerViewToSelect byExtendingSelection:(BOOL)addToSelection;


#pragma mark -
#pragma mark Public calls for controller access
//
// Some of these methods can be called within the class
// But they should still be designed to be used by controller.
//
#pragma mark Legacy from FPModelViewProtocol - no longer existing

- (void)setImagePathForSelectedItems:(NSString *)path;
//- (void)setAlternativeRowForSelectedItems:(BOOL)status;
//- (void)setNumberOfColumns:(NSInteger)number;
//- (void)setNumberOfRows:(NSInteger)number;

////Getters global settings only
//- (NSColor *)fillColorForSelectedItems;
//- (float)strokeWidthForSelectedItems;
//- (NSColor *)strokeColorForSelectedItems;

#pragma mark Controller calls to View
// Should always give the controller requesting work.
// item feature changes always applies to selected item(s)

#pragma mark Page & Info
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetDocumentName:(NSString *)name;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToShowElementsFrame:(BOOL)showElementsFrame;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToShowPrintableArea:(BOOL)showPrintableArea;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetPagePrintInfo:(NSPrintInfo *)printInfo;

#pragma mark Profiles
- (BOOL)controllerWantsToRemoveProfile:(NSObject <FPTemplateViewControllerProtocol>*)controller;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetPagination:(NSInteger)pagination;

#pragma mark Graphics
- (BOOL)controller:(NSObject<FPTemplateViewControllerProtocol> *)controller wantsToFillWithColor:(BOOL)fillWithColor;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetFillingColor:(NSColor *)fillingColor;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetBorderType:(NSInteger)borderType;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetBorderThickness:(CGFloat)thickness;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetBorderColor:(NSColor *)borderColor;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToShowShadow:(BOOL)showShadow;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetShadowColor:(NSColor *)color;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetShadowAngle:(CGFloat)angle;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetOpacity:(CGFloat)opacity;

#pragma mark Metrics
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetWidth:(CGFloat)width;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetHeight:(CGFloat)height;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetX:(CGFloat)x;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetY:(CGFloat)y;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetRotation:(CGFloat)rotation;

#pragma mark Text
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetFontUsingFontManager:(NSFontManager *)fontManager; //should be called from changeFont:
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToSetTextAttributesUsingFontManager:(NSFontManager *)fontManager; // should be called from changeAttributes:
- (BOOL)controller:(NSObject<FPTemplateViewControllerProtocol> *)controller wantsToSetTextAlignment:(NSTextAlignment)textAlignment;

#pragma mark Table
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToAddRowAtIndex:(NSInteger)index; // -1 means at the end
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToDeleteRowAtIndex:(NSInteger)index; // -1 means remove last.
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToAddColumnAtIndex:(NSInteger)index; // -1 means at the end.
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToRemoveColumnAtIndex:(NSInteger)index; // -1 means latest one.
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToUseAlternativeColors:(BOOL)useAlternativeColors;

#pragma mark Layer
- (void)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller didSelectSubView:(NSView *)subview;
- (void)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller didDeselectSubView:(NSView *)subview; // nil means all.
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToMoveSubview:(NSView *)subview toIndex:(NSInteger)index;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToDeleteSubView:(NSView *)subview;
- (BOOL)controller:(NSObject <FPTemplateViewControllerProtocol>*)controller wantsToCreateSubViewWithShapeType:(FPShapesTypes)type withObjectInfo:(id)object;

@end
